//Created by Amin Rahmani, Ph.D., P.E. - February 6, 2018
//Copyright © 2018 Amin Rahmani. All rights reserved.

//==========================================================================================================================
//                                '****   Notes  ****
// This ".cpp" file calculates acceleration response spectrum of an input ground motion in "g".
// This file reads the input acceleration time history, use Newmark-beta method to calculate spectral acceleration for different system periods, and finally writes the results in a user-defined file. 
// The file is a child class for "Spectrum.cpp".
//===========================================================================================================================

#ifndef Spectrum_hpp
#define Spectrum_hpp

#include <iostream>
#include <string>
#include <cmath>
#include <fstream>
#include <vector>

#include <stdio.h>

using namespace std;

class Spectrum {
    
protected:
    //Member variables
    double dt;                  // input motion time step (second)
    const double gamma = 0.50;  // Newamrk Parameter ==> gamma
    const double beta  = 0.25;  // Newamrk Parameter ==> beta
    double zeta;                // damping ratio in %
    const double maxT = 10.0;   // Maximum period (second)
    const double dT   = 0.02;   // Maximum period step (second)
    double gravity;             // gravity in metric (m/s/s) or english units (ft/s/s)
    int numData;                // Number of input acceleration points
    double omega;               // Natural frequency (1/second)
    int numSpect;               // Maximum number of ARS points
    
    string motionDir;            // Input motion directory, accel must be in "g"
    string outputDir;            // Outputs directory
    
    vector <double>  Time;              // Time increments
    vector <double>  ddu;               // Relative acceleration
    vector <double>  ddu_total;         // Absolute accelarion
    vector <double>  ddug;              // Input ground acceleration
    vector <double>  du;                // Relative velocity
    vector <double>  u;                 // Relative displacement
    vector <double>  Sa;                // Spectral acceleration
    vector <double>  Period;            // Period

public:
    //Default constructor
    Spectrum();
    
    //Destructor
    ~Spectrum();
    
    //Set values
    void setValues (double, double, double, string, string);
    
    //Main function to calculate and plot ARS
    void readfiles () ;
    void computeARS () ;
    void writefiles () ;

};

#endif /* Spectrum_hpp */
