//Created by Amin Rahmani, Ph.D., P.E. - February 6, 2018
//Copyright © 2018 Amin Rahmani. All rights reserved.

//==========================================================================================================================
//                                '****   Notes  ****
// This ".cpp" file calculates acceleration, velocity, and displacement time histories for a given SDOF subjected to a given earthquake motion.
// Mass can be given either in kg or in lbs. Stiffness can be given either in N/m or in lbf/ft.
// This file reads some variables and functions from parent class "Spectrum.cpp".
//===========================================================================================================================

#include "TimeHistory.hpp"

timeHistory::timeHistory () {
    // Do nothing
}

timeHistory::~timeHistory () {
    // Do nothing
}

//Set values
void timeHistory::setValues ( double stiffness, double mass, double dt, double zeta, double gravity, string motionDir, string outputDir ) {

    this-> stiffness =stiffness;
    this-> mass =mass;
    this-> dt =dt;
    this-> zeta =zeta;   // in percent
    this-> gravity =gravity;
    this-> motionDir =motionDir;
    this-> outputDir =outputDir;
}

// This function calculates time histories of absolute acceleration, relative velocity and displacement
void timeHistory::computeTH ( ) {

    //reads input file from the parent class "Spectrum"
    readfiles ();

    double omega =sqrt(stiffness/mass);
    double maxvalue=0.0;

    ddu.push_back(0.0);
    du.push_back(0.0);
    u.push_back(0.0);
    ddu_total.push_back(0.0);

    for (int i =0; i< numData; i++) {

        double term1 = -ddug[i+1] - (pow (omega,2.0)*(u[i]+dt*du[i]+(1-2*beta)/2.0 * pow (dt,2.0) * ddu[i])) -(2.*zeta/100.0*omega*(du[i] + (1-gamma)*dt * ddu[i]));
        double term2 = 1.0 + pow (omega,2.0)*beta*pow (dt,2.0) + 2.*zeta/100.0*omega*gamma*dt;

        ddu.push_back(term1/term2);                 //relative acceleration
        ddu_total.push_back(ddu[i+1] + ddug[i+1]);  //absolute acceleration

        du.push_back(du[i] + (1-gamma)*dt * ddu[i]+ gamma*dt*ddu[i+1]);
        u.push_back(u[i]+dt*du[i]+(1-2*beta)/2.0 * pow (dt,2.0) * ddu[i]+ beta*pow (dt,2.0)*ddu[i+1]);

        if ( abs(ddu_total[i+1]) > maxvalue ) {
            maxvalue=abs(ddu_total[i+1]);
        }

    }
    writefiles_TH ( );
}

// This function writes  accel, velocity and displacement time histories in a user-defined file
void timeHistory::writefiles_TH ( ) {

    ofstream outputData;
    outputData.open (outputDir);

    outputData << "Time(sec)" << " " << "Accel(m/s2,ft/s2)" << " " << "Vel.(m/s,ft/s)" << " " << "Disp.(m,ft)" << endl;

    for (int i=0; i < numData; i++) {
        outputData << Time[i] << " " << ddu_total[i] << " " << du[i] << " " << u[i] << endl;
    }

    outputData.close();

}
